package vixid.library;

import processing.core.*;

public class Track{

	Vixid vixid;
	int trackId;
	boolean[] hasVideoIn = { false, false, false, false };
	
	float cropL = 0; // 0 <-> 100%
	float cropR = 0;
	float cropU = 0;
	float cropD = 0;
	int scrollX = 0; // -720 <-> 720;
	int scrollY = 0; // -576 <-> 576;
	int blowL = 160; // 0 <-> 718;
	int blowR = 160; 
	int blowU = 100; // 0 <-> 574;
	int blowD = 100;
	int tolerance = 60; // 0 <-> 443;
	int transition = 5;
	/* 
	 * TODO: Find inital values and range for pipetteX and pipetteY in the Keyer.
	 */
	float pipetteX;
	float pipetteY;

	int inputSelected = 0; // 0 <-> 3 | 0 : input1, 1 : input 2,...

	boolean solo = false;
	boolean mute = false;
	boolean negative = false;
	boolean b_w = false;
	boolean bkgAlpha = false;
	int opacity = 0; // possibly only 128 values
	int gain; // 0 <-> 250
	int blendMode = 0; // 0 - 12 | 0 : normal, 1 : Additive,...   
	int layer; // 0 <-> 3 | 0 : layer A, 1 : layer B,...

	int redGain = 128; // 0 <-> 255
	int greenGain = 128;
	int blueGain = 128;
	int redOffset = 0; // -255 <-> 255
	int greenOffset = 0;
	int blueOffset = 0;
	int brightness = 128; // 0 <-> 255
	int contrast = 128;
	int saturation = 128;

	boolean motionFx = false;
	int motionFxType = 0; // 0 <-> 3 | 1 : Freeze, 2 : SlowMo,...
	int alphaPeriod = 5; // 0 <-> 150/period
	int period = 25; // 0 <-> 150
	int blocParamX = 3; // 1 <-> 7
	int blocParamY = 3;

	boolean crop = false;
	int cropType = 0; // 0 : Crop, 1 : CropCenter

	boolean keyer = false;
	int keyerType = 0; // 0 <-> 5 | 0 : Color keyer, 1 : Chroma keyer,...
	int trackIn; // 1 <-> 4 | 1 : Track 1, 2 : Track 2,...
	int pre_post = 1; // 0 : PreFx, 1 : PostFx
	boolean inverse = false;
	int color1; // 0 <-> 255 | Red / Y
	int color2;  
	int color3;
	int pickMode = 1; // 0 : Map mode, 1: Picker mode 
	boolean pickerState = false;

	boolean fx = false;
	int fxType; // 0 <-> 14 | 0 : MirrorL, 1 : MirrorR,...
	int zoomX = 1; // 1 <-> 63
	int zoomY = 1; 
	int mosaicZoomX = 7;
	int mosaicZoomY = 7;


	public Track(Vixid v, int nb, boolean[] videoIn ){
		vixid = v;
		trackId = nb;
		for(int i=0; i < hasVideoIn.length; i++){
			hasVideoIn[i] = videoIn[i];
		}
		reset();
	}
	
	public void setCropL(float value){
		if((value >= 0) && (value <=100)){
			cropL = value;
			sendMidi(0, PApplet.map(cropL, 0, 100, 0, 127));
		}
	}
	
	public void setCropR(float value){
		if((value >= 0) && (value <=100)){
			cropR = value;
			sendMidi(1, PApplet.map(cropR, 0, 100, 0, 127));
		}
	}
	
	public void setCropU(float value){
		if((value >= 0) && (value <=100)){
			cropU = value;
			sendMidi(2, PApplet.map(cropU, 0, 100, 0, 127));
		}
	}
	
	public void setCropD(float value){
		if((value >= 0) && (value <=100)){
			cropD = value;
			sendMidi(3, PApplet.map(cropD, 0, 100, 0, 127));
		}
	}
	
	public void setScrollX(int value){
		if((value >= -720) && (value <=720)){
			scrollX = value;
			sendMidi(4, PApplet.map(scrollX, -720, 720, 0, 127));
		}
	}
	
	public void setScrollY(int value){
		if((value >= -576) && (value <=576)){
			scrollY = value;
			sendMidi(5, PApplet.map(scrollY, -576, 576, 0, 127));
		}
	}
	
	public void setScrollXY(int x, int y){
		setScrollX(x);
		setScrollY(y);
	}
	
	public void setTolerance(int value ){
		if((value>=0) && (value <=443)){
			tolerance = value;
			sendMidi(10, PApplet.map(tolerance, 0, 443, 0, 128));
		}
	}
	
	public void setTransition(int value ){
		if((value>=0) && (value <=443)){
			transition = value;
			sendMidi(11, PApplet.map(transition, 0, 443, 0, 128));
		}
	}
	
	
	/**
	 * Select the video input for this track if there is video feed in.
	 * @param inputId
	 * 		0 : input1 (COMP), 1 : input2 (COMP), 2 : input3 (S-VIDEO), 3 : input4 (S-VIDEO)
	 */
	public void selectVideoInput(int inputId){
		if((inputId >=0) && (inputId <= 3)){
			if(hasVideoIn[inputId]){
				// if there is video feed in that input
				inputSelected = inputId;
				sendMidi(64, inputId*32);
				PApplet.println("track: " + trackId + " input: " + inputId);
			}
		}
	}

	public void setSoloOn(){
		solo = true;
		sendMidi(65, 120);
	}

	public void setSoloOff(){
		solo = false;
		sendMidi(65, 0);
	}

	public void setSolo(boolean solo){
		if(solo) {
			setSoloOn();
		} else {
			setSoloOff();
		}
	}
	
	public void setMuteOn(){
		mute = true;
		sendMidi(66, 120);
	}
	
	public void setMuteOff(){
		mute = false;
		sendMidi(66, 0);
	}

	public void setMute(boolean mute){
		if( mute ){
			setMuteOn();
		} else {
			setMuteOff();
		}
	}
	
	public void setNegativeOn(){
		negative = true;
		sendMidi(67, 120);
	}

	public void setNegativeOff(){
		negative = false;
		sendMidi(67, 0);
	}
	
	public void setNegative(boolean negative){
		if( negative ){
			setNegativeOn();
		} else {
			setNegativeOff();
		}
	}
	
	public void setBlackAndWhiteOn(){
		b_w = true;
		sendMidi(68, 120);
	}
	
	public void setBlackAndWhiteOff(){
		b_w = false;
		sendMidi(68, 0);
	}
	
	public void setBlackAndWhite(boolean bw){
		if( bw ) {
			setBlackAndWhiteOn();
		} else {
			setBlackAndWhiteOff();
		}
	}
	
	public void setBkgAlphaOn(){
		bkgAlpha = true;
		sendMidi(69, 120);
	}
	
	public void setBkgAlphaOff(){
		bkgAlpha = false;
		sendMidi(69, 0);
	}
	
	public void setBkgAlpha(boolean transparent){
		if( transparent ){
			setBkgAlphaOn();
		} else {
			setBkgAlphaOff();
		}
	}
	
	public void setOpacity(){
		//TODO: write SetOpacity() function
	}
	
	public void getOpacity(){
		//TODO: write getOpacity() function
	}
	
	public void setGain(){
		//TODO: write SetGain() function
	}
	
	public void getGain(){
		//TODO: write getGain() function
	}
	
	/**
	 * Set blend mode to be normal, additive, average,...
	 * 
	 * @param mode
	 * 		0 : normal, 1 : additive, 2 : average, 3 : darken, 4 : lighten, 5 : stamp, 6 : diff, 7 : substract, 8 : negation, 9 : xor, 10 : red, 11 : green, 12 : blue <br>
	 * 		Any number out of this range will set mode to normal. 
	 * 		
	 */
	public void setBlendMode(int mode){
		if((mode >= 0) && (mode <= 12)){
			blendMode = mode;
			sendMidi(72, mode*8);
		} else {
			setBlendMode(0);
		}
	}
	
	public void setLayer(int layerId){
		//TODO: This setLayer() method is incomplete as it doesn't modify the other tracks layer as the Vixid does it.
		if((layerId >=0) && (layerId<=3)){
			layer = layerId;
			sendMidi(73, layer*32);
		}
	}
	
	/**
	 * Set the red gain in the RGB effects.
	 * 
	 * @param value
	 * 		set it to be between 0 and 255 or it will do nothing
	 */
	public void setRedGain(int value) {
		if((value >= 0) && (value <= 255)){
			redGain = value;
			sendMidi(75, PApplet.map(value, 0, 255, 0, 127));
		}
	}
	
	/**
	 * Set the green gain in the RGB effects.
	 * 
	 * @param value
	 * 		set it to be between 0 and 255 or it will do nothing
	 */
	public void setGreenGain(int value) {
		if((value >= 0) && (value <= 255)){
			greenGain = value;
			sendMidi(76, PApplet.map(value, 0, 255, 0, 127));
		}
	}
	
	/**
	 * Set the blue gain in the RGB effects.
	 * 
	 * @param value
	 * 		set it to be between 0 and 255 or it will do nothing
	 */
	public void setBlueGain(int value) {
		if((value >= 0) && (value <= 255)){
			blueGain = value;
			sendMidi(77, PApplet.map(value, 0, 255, 0, 127));
		}
	}
	
	/**
	 * Set the RGB gain in the RGB effects.
	 * 
	 * @param r
	 * @param g
	 * @param b
	 * 
	 */
	public void setRGBGain(int r, int g, int b) {
		setRedGain(r);
		setGreenGain(g);
		setBlueGain(b);
	}
	
	/**
	 * Set the red offset in the RGB effects.
	 * 
	 * @param value
	 * 		set it to be between -255 and 255 or it will do nothing
	 */
	public void setRedOffset(int value) {
		if((value >= -255) && (value <= 255)){
			redOffset = value;
			sendMidi(78, PApplet.map(value, -255, 255, 0, 127));
		}
	}
	
	/**
	 * Set the green offset in the RGB effects.
	 * 
	 * @param value
	 * 		set it to be between -255 and 255 or it will do nothing
	 */
	public void setGreenOffset(int value) {
		if((value >= -255) && (value <= 255)){
			greenOffset = value;
			sendMidi(79, PApplet.map(value, -255, 255, 0, 127));
		}
	}
	
	/**
	 * Set the blue offset in the RGB effects.
	 * 
	 * @param value
	 * 		set it to be between -255 and 255 or it will do nothing
	 */
	public void setBlueOffset(int value) {
		if((value >= -255) && (value <= 255)){
			blueOffset = value;
			sendMidi(80, PApplet.map(value, -255, 255, 0, 127));
		}
	}
	
	/**
	 * Set the RGB offset in the RGB effects.
	 * 
	 * @param r
	 * @param g
	 * @param b
	 * 
	 */
	public void setRGBOffset(int r, int g, int b) {
		setRedOffset(r);
		setGreenOffset(g);
		setBlueOffset(b);
	}
	
	
	
	/**
	 * Set the brightness in the BCS effects.
	 * 
	 * @param value
	 * 		set it to be between 0 and 255 or it will do nothing
	 */
	public void setBrightness(int value) {
		if((value >= 0) && (value <= 255)){
			brightness = value;
			sendMidi(81, PApplet.map(value, 0, 255, 0, 127));
		}
	}
	
	/**
	 * Set the contrast in the BCS effects.
	 * 
	 * @param value
	 * 		set it to be between 0 and 255 or it will do nothing
	 */
	public void setContrast(int value) {
		if((value >= 0) && (value <= 255)){
			contrast = value;
			sendMidi(82, PApplet.map(value, 0, 255, 0, 127));
		}
	}
	
	/**
	 * Set the saturation in the BCS effects.
	 * 
	 * @param value
	 * 		set it to be between 0 and 255 or it will do nothing
	 */
	public void setSaturation(int value) {
		if((value >= 0) && (value <= 255)){
			saturation = value;
			sendMidi(83, PApplet.map(value, 0, 255, 0, 127));
		}
	}
	
	/**
	 * Set brightness, contrast and saturation in the BCS effects.
	 * 
	 * @param b
	 * @param c
	 * @param s
	 * 	
	 */
	public void setBCS(int b, int c, int s) {
		setBrightness(b);
		setContrast(c);
		setSaturation(s);
	}
	
	//TODO: write motion effects functions
	
	/**
	 * Set crop to these L, R, U, D values between 0 and 100%
	 * 
	 * @param left
	 * @param right
	 * @param up
	 * @param down
	 * 
	 */
	public void setCrop(float left, float right, float up, float down){
		setCropL(left);
		setCropR(right);
		setCropU(up);
		setCropD(down);
	}
	
	/**
	 * Set crop and crop type together.
	 * L, R, U, D values are between 0 and 100%
	 * 
	 * @param left
	 * @param right
	 * @param up
	 * @param down
	 * @param type
	 * 		0 : crop, 1 : crop center
	 * 
	 */
	public void setCrop(float left, float right, float up, float down, int type){
		setCropType(type);
		setCrop(left, right, up, down);
	}
	

	public void setCropOn(){
		crop = true;
		sendMidi(90, 120);
	}
	
	public void setCropOff(){
		crop = false;
		sendMidi(90, 0);
	}
	
	public void setCrop(boolean crop){
		if( crop ){
			setCropOn();
		} else {
			setCropOff();
		}
	}
	
	public void setCropType(int type){
		if( type == 0 ){
			// crop
			cropType = 0;
			sendMidi(91, 0);
		}
		if( type == 1 ){
			// crop center
			cropType = 1;
			sendMidi(91, 120);
		}
		
	}
	
	public void setKeyerOn(){
		keyer = true;
		sendMidi(102, 120);
	}
	
	public void setKeyerOff(){
		keyer = false;
		sendMidi(102, 0);
	}
	
	public void setKeyer(boolean keyer){
		if ( keyer ){
			setKeyerOn();
		} else {
			setKeyerOff();
		}
	}
	
	/**
	 * Set the keyer type.
	 * 
	 * @param type
	 * 		0 : color key, 1 : chroma key, 2 : luma key, 3 : red key, 4 : green key, 5 : blue key
	 */
	public void setKeyerType(int type){
		if((type >= 0) && (type <= 5)){
			keyerType = type;
			sendMidi(103, keyerType*16);
		}
	}
	
	/**
	 * Set the Track number for the keyer input
	 * @param trackId
	 * 		0 : track 1, 1 : track 2, 2 : track 3, 3 : track 4
	 */
	public void setTrackIn(int trackId){
		if((trackId >= 0) && (trackId <= 3)){
			trackIn = trackId;
			sendMidi(104, trackIn*32);
		}
	}
	
	/**
	 * Set Pre or Post Fx for the Keyer
	 * 
	 * @param pp
	 * 		0 : Pre Fx, 1 : Post Fx
	 */
	public void setPrePost(int pp){
		if(pp == 0){
			// PreFx keyer
			pre_post = 0;
			sendMidi(105, 0);
		}
		if(pp == 1){
			// PostFx keyer
			pre_post = 1;
			sendMidi(105, 120);
		}
	}
	
	/**
	 * Switch on the keyer inverse 
	 */
	public void setInverseOn(){
		inverse = true;
		sendMidi(106, 120);
	}
	
	/**
	 * Switch off the keyer inverse
	 */
	public void setInverseOff(){
		inverse = false;
		sendMidi(106, 0);
	}
	
	public void setInverse(boolean inverse){
		if( inverse ){
			setInverseOn();
		} else {
			setInverseOff();
		}
	}
	
	//TODO: write keyer color functions
	
	/**
	 * Set the keyer pick mode
	 * @param mode
	 * 		0 : Map mode, 1 : Picker Mode
	 */
	public void setPickMode(int mode){
		if( mode == 0) {
			// Map mode
			pickMode = 0;
			sendMidi(110, 0);
		}
		if( mode == 1) {
			//Picker mode
			pickMode = 1;
			sendMidi(110, 120);
		}
	}
	
	/**
	 * Enable the keyer picker state
	 */
	public void setPickerStateOn(){
		pickerState = true;
		sendMidi(111, 120);
	}
	
	/**
	 * Disable the keyer picker state
	 */
	public void setPickerStateOff(){
		pickerState = false;
		sendMidi(111, 0);
	}
	
	
	public void setPickerState(boolean pickerState){
		if( pickerState ){
			setPickerStateOn();
		} else {
			setPickerStateOff();
		}
	}
		
	public void setFxOn(){
		fx = true;
		sendMidi(112, 120);
	}
	
	public void setFxOff(){
		fx = false;
		sendMidi(112, 0);
	}
	
	public void setFx(boolean fx){
		if( fx ){
			setFxOn();
		} else {
			setFxOff();
		}
	}
	
	/**
	 * Set the effects to be type of:
	 * @param type
	 * 		0 : MirrorL, 1 : MirrorR, 2 : MirrorU, 3 : MirrorD, 4: FlipH, 5 : FlipV, 6 : Rot180, 7 : Blow, 8 : Scroll, 9 : Scrollwrap, 10 : ScrollX, 11 : ScrollY, 12 : ScrollXWrap, 13 : ScrollYWrap, 14 : Mosaic 
	 */
	public void setFxType(int type){
		if((type >=0) && (type <=14)){
			fxType = type;
			sendMidi(113, type*8);
		}
	}
	
	/**
	 * Set the x parameter for the zoom effect
	 * @param zoom
	 * 		values are between 1 and 63
	 */
	public void setZoomX(int zoom){
		if((zoom >= 1) && (zoom <= 63)){
			zoomX = zoom;
			//sendMidi(114, PApplet.map(zoomX, 1, 63, 0, 128));
			sendMidi(114, zoomX*2);
		}
	}
	
	/**
	 * Set the y parameter for the zoom effect
	 * @param zoom
	 * 		values are between 1 and 63
	 */
	public void setZoomY(int zoom){
		if((zoom >= 1) && (zoom <= 63)){
			zoomY = zoom;
			//sendMidi(115, PApplet.map(zoomY, 1, 63, 0, 128));
			sendMidi(115, zoomY*2);
		}
	}
	
	public void setZoomXY(int x, int y){
		// not using the dedicated midi channel
		setZoomX(x);
		setZoomY(y);
	}
	
	/**
	 * Set the x parameter for the mosaic zoom effect
	 * @param zoom
	 * 		values are between 1 and 63
	 */
	public void setMosaicZoomX(int zoom){
		if((zoom >= 1) && (zoom <= 63)){
			mosaicZoomX = zoom;
			sendMidi(117, mosaicZoomX*2);
		}
	}
	
	/**
	 * Set the y parameter for the mosaic zoom effect
	 * @param zoom
	 * 		values are between 1 and 63
	 */
	public void setMosaicZoomY(int zoom){
		if((zoom >= 1) && (zoom <= 63)){
			mosaicZoomY = zoom;
			sendMidi(118, mosaicZoomY*2);
		}
	}
	
	public void setMosaicZoomXY(int x, int y){
		// not using the dedicated midi channel for this
		setMosaicZoomX(x);
		setMosaicZoomY(y);
	}
	
	/**
	 * Reset the track to it's default values. 
	 * Should be the same as pressing  the reset button and track button together.
	 * Needs to be perfected...
	 */
	//TODO: Rewrite this track reset function to correspond to what Vixid does
	public void reset(){
		setCrop(0, 0, 0, 0, 0);
		setTolerance(60);
		setTransition(5);
		selectVideoInput(0);
		setSolo(false);
		setMute(false);
		setNegative(false);
		setBlackAndWhite(false);
		setBkgAlpha(false);
		setBlendMode(0);
		setLayer(trackId);
		setRGBGain(128, 128, 128);
		setRGBOffset(0, 0, 0);
		setBCS(128, 128, 128);
		setCrop(false);
		setCropType(0);
		setFx(false);
		setFxType(0);
		setTrackIn(trackId);
		setPrePost(1);
		setInverse(false);
		setPickMode(1);
		setPickerState(false);
		setFx(false);
		setZoomXY(1, 1);
		setMosaicZoomXY(7, 7);
	}
	
	private void sendMidi(int channel, int value) {
		vixid.midibus.sendControllerChange(trackId, channel, value);
	}
	
	private void sendMidi(int channel, float value) {
		vixid.midibus.sendControllerChange(trackId, channel, (int)(value));
	}
	
}